@@ -19,7 +19,7 @@
  * along with "Otags reloaded". If not, see
  * <http://www.gnu.org/licenses/>.
  * 
- * $Id: tags.ml,v 1.26 2012-05-22 06:16:47 tews Exp $
+ * $Id: tags.ml,v 1.29 2012/12/05 09:29:31 tews Exp $
  * 
  * recursive tagging function
  * 
@@ -54,6 +54,13 @@ let tag_constructor_decl write_tag = function
      *)
     let wrong_id_loc = translate_loc (Ast.loc_of_ident id) in
     write_tag (Reparse.loc_of_first_word wrong_id_loc) uid
+  (* the next case is for GADT constructors in OCaml 4.00 *)
+  | <:ctyp< $id:<:ident< $uid:uid$>> as id$ : $_$ >> ->
+    (* Wrong location info in id: XXX not reported yet
+     * We need the first word
+     *)
+    let wrong_id_loc = translate_loc (Ast.loc_of_ident id) in
+    write_tag (Reparse.loc_of_first_word wrong_id_loc) uid
   | _ -> assert false
 
 
@@ -145,6 +152,9 @@ let rec tag_type write_tag typ = match typ with
   | <:ctyp< + ' $_$ >>			(* type parameter *)
   | <:ctyp< - ' $_$ >>			(* type parameter *)
   | <:ctyp< _ >>			(* anonymous type variable *)
+  | Ast.TyAnP _				(* no syntax for + _ *)
+  | Ast.TyAnM _				(* no syntax for - _ *)
+  | Ast.TyTypePol _			(* no syntax for type x . type-expr *)
   | <:ctyp< $anti:_$ >>
       -> assert false
 
@@ -511,6 +521,13 @@ let rec tag_let_pattern write_tag = function
     in
     write_tag loc lid;
     (* Printf.eprintf "TAGPATTEND\n%!" *)
+
+  (* let matching with constant constructors: let A = ... in ... *)
+  | <:patt< $uid:_$ >>
+  | <:patt< $id: <:ident< $_$ . $_$ >> $ >>
+  | <:patt< $id: <:ident< $anti:_$ >> $ >> -> ()
+  | <:patt< $id: <:ident< $_$ $_$ >> $ >> -> assert false
+
   | <:patt< ( $tup:p1$ ) >> ->
     List.iter
       (tag_let_pattern write_tag)
@@ -543,8 +560,34 @@ let rec tag_let_pattern write_tag = function
   | <:patt< ($p1$ : $_$) >> ->
     tag_let_pattern write_tag p1
 
-  (* XXX catch all *)
-  | _ -> ()
+  | <:patt< $chr:_$ >>
+  | <:patt< $int:_$ >>
+  | <:patt< $int32:_$ >>
+  | <:patt< $int64:_$ >>
+  | <:patt< $nativeint:_$ >>
+  | <:patt< $str:_$ >>
+  | <:patt< $flo:_$ >>
+  | <:patt< [| $_$ |] >>
+  | <:patt< { $_$ } >>
+  | <:patt< ` $_$ >>
+  | <:patt< # $_$ >>
+  | <:patt< $_$ .. $_$ >>
+  | <:patt< $_$ | $_$ >>
+  | <:patt< $_$ $_$ >>			(* eg, constructor application :: *)
+  | <:patt< _ >>
+  | <:patt< lazy $_$ >>
+  | <:patt< $anti:_$ >>
+    -> ()
+
+  | <:patt< $_$ = $_$ >> 		(* only inside records *)
+  | <:patt< ~ $_$ : $_$ >>	        (* only inside fun expressions *)
+  | <:patt< ? $_$ : ( $_$ ) >>	        (* only inside fun expressions *)
+  | <:patt< ? $_$ : ( $_$ = $_$ ) >>	(* only inside fun expressions *)
+  | <:patt< $_$ ; $_$ >>		(* only inside arrays, records *)
+  | <:patt< $_$ , $_$ >>		(* handled by Ast.list_of_patt *)
+  | <:patt< ( module $_$ ) >>  (* package pattern not permitted in top-level *)
+  | <:patt< >> 
+    -> assert false
 
 
 
